def SetDHWTemp(current,new):
    try:
        newtemp = []
        newtemp.append(current[0]&255)
        newtemp.append(current[1])
        newtemp.append(current[2])
        newtemp.append(current[3]&15)
        newtemp.append(512 + (current[4]&255))
        newtemp.append(int(new * 2)<<8 | (current[5]&255))
        return newtemp
    except ValueError:
        return "ERROR"



def GetCompInfo(payload):
    """
    Thanks to Daniel Mentel
    Function for displaying compressor paramameters(set frequency, actual frequency, compressor current and compressor voltage), return as array
    payload - register from 241 to 261
    :return:
    """
    if len(payload) == 22:
        compressor=[]
        fact = divmod(int(hex(payload[2]), 16), 256)[1]
        fset = divmod(int(hex(payload[3]), 16), 256)[0]
        compcurr = divmod(int(hex(payload[15]), 16), 256)[0]*2/10
        compvolt = divmod(int(hex(payload[15]), 16), 256)[1]*4
        comptemp = divmod(int(hex(payload[14]), 16), 256)[1]/10
        compressor.append(fset)
        compressor.append(fact)
        compressor.append(compcurr)
        compressor.append(compvolt)
        compressor.append(comptemp)
        return compressor
    else:
        return "Bad payload length"



def SetState(current,new):
    """
    function to change pump state

    :param current:
    :param new:
    :return:
    """
    if len(current) !=6:
        return 'Bad current frame length'

    try:
        newstate=[]
        if new == "off":
            state=int(current[0] & 0xff) -1
        elif new == "on":
            state=int(current[0] & 0xff) +1
        elif new == "C":
            state=3
        elif new == "H":
            state=5
        elif new == "T":
            state=135
        elif new == "CT":
            state=131
        elif new == "HT":
            state=133
        else:
            return -1

        if new == "on" or new == "off":
            num=256
        else:
            num=34304
        newstate.append(num+state)
        newstate.append(current[1])
        newstate.append(current[2])
        newstate.append(current[3]&15)
        newstate.append(current[4]&255)
        newstate.append(current[5])


        return newstate
    except:
        return "ERROR"



def GetDefrost(payload):
    """
    Function for displaying if pump is on or off
    payload - register from 141 to 157
    :return:
    """
    if len(payload) == 16:
        if payload[0] & 8192:
            threeway = 'DEFROST'
        else:
            threeway= ''
        return threeway
    else:
        return "Bad payload length"



def GetCHTemp(payload):
    """
    Cenral heating temperature display function
    :return:
    """
    if len(payload) == 6:
        chtemp = int(hex(payload[1])[2:4], 16)/2
        return chtemp
    else:
        return "Bad payload length"



def GetPdPs(payload):
    """
    Thanks to Daniel Mentel
    Function for displaying Pd set and actual and Ts set and actual paramameters, return as array (Pd for now)
    payload - register from 241 to 261
    :return:
    """
    if len(payload) == 22:
        pdps=[]
        pdset = divmod(int(hex(payload[5]), 16), 256)[1]*2/10 #B4D
        pdact = divmod(int(hex(payload[6]), 16), 256)[0]*2/10 #B4E
        psset = divmod(int(hex(payload[8]), 16), 256)[0]*2/10 #B51
        psact = divmod(int(hex(payload[8]), 16), 256)[1]*2/10 #B52

        pdps.append(pdset)
        pdps.append(pdact)
        pdps.append(psset)
        pdps.append(psact)
        return pdps
    else:
        return "Bad payload length"




def SetCHTemp(current,new):
    try:
        newtemp = []
        newtemp.append(1024 + (current[0]&255))
        newtemp.append(int(new * 2)<<8 | (current[1]&255))
        newtemp.append(current[2])
        newtemp.append(current[3]&15)
        newtemp.append(current[4]&255)
        newtemp.append(current[5])
        return newtemp
    except:
        return "ERROR"



def GetError(payload):
    """
    Function for displaying active error, return as int
    payload - register from 141 to 157
    :return:
    """
    if len(payload) == 16:
        error=payload[0] & 0xFF
        return error
    else:
        return "Bad payload length"



def SetHeater(current,new):
    """
    Heater (A04) set function 0=OFF 1=ON
    :return:
    """
    try:
        frame=[]
        pumpstate=current[0]&255
        heaterstate=new*8+pumpstate
        frame.append(4096+heaterstate)
        frame.append(current[1])
        frame.append(current[2])
        frame.append(current[3]&15)
        frame.append(current[4]&255)
        frame.append(current[5])
        return frame
    except:
        return "ERROR"



def GetLastError(payload):
    """
    Function for displaying last known error return int
    payload - register from 241 to 261
    :return:
    """
    if len(payload) == 22:
        error=payload[0] & 0xFF
        return error
    else:
        return "Bad payload length"




def GetState(payload):
    """
    Mode display function
    :return:
    """

    if len(payload) != 6:
        return "Bad payload length"

    state = payload[0]

    if state & 1:
        onoff = 'ON'
    else:
        onoff = 'OFF'

    if state & 128:
        tank = 'TANK '
    else:
        tank = ''

    is_cool = state & 2
    is_heat = state & 4

    heatcool = ''

    if is_cool and is_heat:
        heatcool = ''
    elif is_cool:
        heatcool = 'COOL '
    elif is_heat:
        heatcool = 'HEAT '
    output = f'{heatcool}{tank}{onoff}'.strip()
    return output



def GetMode(payload):
    """
    Mode display function
    :return:
    """
    if len(payload) == 1:
        if payload[0] == 0:
            mode="eco"
        elif payload[0] == 1:
            mode="quiet"
        elif payload[0] == 2:
            mode="turbo"
        else:
            mode="Mode Undefinde"

        return mode
    else:
        return "Bad payload length"



def GetAntifreeze(payload):
    """
    Function for displaying if pump is on or off
    payload - register from 141 to 157
    :return:
    """
    if len(payload) == 16:
        if payload[0] & 2048:
            threeway = 'ANTIFREEZE'
        else:
            threeway = ''
        return threeway
    else:
        return "Bad payload length"



def SetDHWCompensation(current, newcompe):
    """
    DHW Compensation (A0d) set function
    :return:
    """
    try:
        frame=[]
        pumpstate=current[0]&255
        currtemp=current[5] >>8
        compensation=30+newcompe*2
        frame.append(2048+pumpstate)
        frame.append[1]
        frame.append(current[2])
        frame.append(current[3]&15)
        frame.append(current[4]&255)
        frame.append((currtemp <<8)|compensation)
        return frame
    except:
        return "ERROR"



def SetPump(current,new):
    """
    Circulation Pump (A05) set function 0=OFF 1=ON
    :return:
    """
    try:
        frame=[]
        pumpstate=current[0]&255
        cpumpstate=new*32+pumpstate
        frame.append(8192+cpumpstate)
        frame.append(current[1])
        frame.append(current[2])
        frame.append(current[3]&15)
        frame.append(current[4]&255)
        frame.append(current[5])

        return frame
    except:
        return "ERROR"



def GetTdTs(payload):
    """
    Thanks to Daniel Mentel
    Function for displaying Td and Ts paramameters, return as array
    payload - register from 241 to 261
    :return:
    """
    if len(payload) == 22:
        tdts=[]
        tdtsb=[f'{divmod(int(hex(payload[10]),16), 256)[1]:08b}'[4:], f'{divmod(int(hex(payload[10]), 16), 256)[1]:08b}'[0:4]]
        tdb=f'{divmod(int(hex(payload[11]), 16), 256)[0]:08b}'
        tsb=f'{divmod(int(hex(payload[11]), 16), 256)[1]:08b}'
        td = int(str(tdtsb[0])+str(tdb), 2)
        if (td > 2047) :
            td -= 4095
        tdts.append(td/10)
        ts = int(str(tdtsb[1])+str(tsb), 2)
        if (ts > 2047) :
            ts -= 4095
        tdts.append(ts/10)
        return tdts
    else:
        return "Bad payload length"



def GetDHWCompensation(payload):
    """
    DHW? Compensation (A0d) display function
    :return:
    """
    if len(payload) == 6:
        dhwcom= int(int(payload[5] &0xFF)-30)/2
        return dhwcom
    else:
        return "Bad payload length"



def GetTempCompensation(payload):
    """
    Temperature Compensation (A03) display function
    :return:
    """
    if len(payload) == 6:
        tempcom= int(int(payload[1] &0xFF)-30)/2
        return tempcom
    else:
        return "Bad payload length"
